@@ -1,5 +1,5 @@
 # engine/__init__.py
-# Copyright (C) 2005, 2006, 2007, 2008, 2009 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -50,7 +50,9 @@ url.py
     within a URL.
 """
 
-import sqlalchemy.databases
+# not sure what this was used for
+#import sqlalchemy.databases  
+
 from sqlalchemy.engine.base import (
     BufferedColumnResultProxy,
     BufferedColumnRow,
@@ -58,7 +60,6 @@ from sqlalchemy.engine.base import (
     Compiled,
     Connectable,
     Connection,
-    DefaultRunner,
     Dialect,
     Engine,
     ExecutionContext,
@@ -66,9 +67,9 @@ from sqlalchemy.engine.base import (
     ResultProxy,
     RootTransaction,
     RowProxy,
-    SchemaIterator,
     Transaction,
-    TwoPhaseTransaction
+    TwoPhaseTransaction,
+    TypeCompiler
     )
 from sqlalchemy.engine import strategies
 from sqlalchemy import util
@@ -81,7 +82,6 @@ __all__ = (
     'Compiled',
     'Connectable',
     'Connection',
-    'DefaultRunner',
     'Dialect',
     'Engine',
     'ExecutionContext',
@@ -89,9 +89,9 @@ __all__ = (
     'ResultProxy',
     'RootTransaction',
     'RowProxy',
-    'SchemaIterator',
     'Transaction',
     'TwoPhaseTransaction',
+    'TypeCompiler',
     'create_engine',
     'engine_from_config',
     )
@@ -107,10 +107,11 @@ def create_engine(*args, **kwargs):
     arguments sent as options to the dialect and resulting Engine.
 
     The URL is a string in the form
-    ``dialect://user:password@host/dbname[?key=value..]``, where
-    ``dialect`` is a name such as ``mysql``, ``oracle``, ``postgres``,
-    etc.  Alternatively, the URL can be an instance of
-    :class:`~sqlalchemy.engine.url.URL`.
+    ``dialect+driver://user:password@host/dbname[?key=value..]``, where
+    ``dialect`` is a database name such as ``mysql``, ``oracle``, 
+    ``postgresql``, etc., and ``driver`` the name of a DBAPI, such as 
+    ``psycopg2``, ``pyodbc``, ``cx_oracle``, etc.  Alternatively, 
+    the URL can be an instance of :class:`~sqlalchemy.engine.url.URL`.
 
     `**kwargs` takes a wide variety of options which are routed 
     towards their appropriate components.  Arguments may be 
@@ -118,13 +119,13 @@ def create_engine(*args, **kwargs):
     Pool.  Specific dialects also accept keyword arguments that
     are unique to that dialect.   Here, we describe the parameters
     that are common to most ``create_engine()`` usage.
-    
-    :param assert_unicode=False: When set to ``True`` alongside
-        convert_unicode=``True``, asserts that incoming string bind
-        parameters are instances of ``unicode``, otherwise raises an
-        error. Only takes effect when ``convert_unicode==True``. This
-        flag is also available on the ``String`` type and its
-        descendants. New in 0.4.2.
+
+    :param assert_unicode:  Deprecated.  A warning is raised in all cases when a non-Unicode
+        object is passed when SQLAlchemy would coerce into an encoding
+        (note: but **not** when the DBAPI handles unicode objects natively).
+        To suppress or raise this warning to an 
+        error, use the Python warnings filter documented at:
+        http://docs.python.org/library/warnings.html
 
     :param connect_args: a dictionary of options which will be
         passed directly to the DBAPI's ``connect()`` method as
@@ -150,14 +151,13 @@ def create_engine(*args, **kwargs):
         ``Engine`` can be modified at any time to turn logging on and
         off. If set to the string ``"debug"``, result rows will be
         printed to the standard output as well. This flag ultimately
-        controls a Python logger; see `dbengine_logging` at the end of
-        this chapter for information on how to configure logging
-        directly.
+        controls a Python logger; see :ref:`dbengine_logging` for
+        information on how to configure logging directly.
 
     :param echo_pool=False: if True, the connection pool will log
         all checkouts/checkins to the logging stream, which defaults to
         sys.stdout. This flag ultimately controls a Python logger; see
-        `dbengine_logging` for information on how to configure logging
+        :ref:`dbengine_logging` for information on how to configure logging
         directly.
 
     :param encoding='utf-8': the encoding to use for all Unicode
@@ -169,11 +169,25 @@ def create_engine(*args, **kwargs):
         characters. If less than 6, labels are generated as
         "_(counter)". If ``None``, the value of
         ``dialect.max_identifier_length`` is used instead.
+    
+    :param listeners: A list of one or more 
+        :class:`~sqlalchemy.interfaces.PoolListener` objects which will 
+        receive connection pool events.
+      
+    :param logging_name:  String identifier which will be used within
+        the "name" field of logging records generated within the
+        "sqlalchemy.engine" logger. Defaults to a hexstring of the 
+        object's id.
+
+    :param max_overflow=10: the number of connections to allow in
+        connection pool "overflow", that is connections that can be
+        opened above and beyond the pool_size setting, which defaults
+        to five. this is only used with :class:`~sqlalchemy.pool.QueuePool`.
 
     :param module=None: used by database implementations which
         support multiple DBAPI modules, this is a reference to a DBAPI2
         module to be used instead of the engine's default module. For
-        Postgres, the default is psycopg2. For Oracle, it's cx_Oracle.
+        PostgreSQL, the default is psycopg2. For Oracle, it's cx_Oracle.
 
     :param pool=None: an already-constructed instance of
         :class:`~sqlalchemy.pool.Pool`, such as a
@@ -190,10 +204,10 @@ def create_engine(*args, **kwargs):
         instantiate the pool in this case, you just indicate what type
         of pool to be used.
 
-    :param max_overflow=10: the number of connections to allow in
-        connection pool "overflow", that is connections that can be
-        opened above and beyond the pool_size setting, which defaults
-        to five. this is only used with :class:`~sqlalchemy.pool.QueuePool`.
+    :param pool_logging_name:  String identifier which will be used within
+       the "name" field of logging records generated within the 
+       "sqlalchemy.pool" logger. Defaults to a hexstring of the object's 
+       id.
 
     :param pool_size=5: the number of connections to keep open
         inside the connection pool. This used with :class:`~sqlalchemy.pool.QueuePool` as
