@@ -52,7 +52,7 @@ decoration::
   class InstrumentedList(list):
       pass
 
-Collection classes can be specified in ``relation(collection_class=)`` as
+Collection classes can be specified in ``relationship(collection_class=)`` as
 types or a function that returns an instance.  Collection classes are
 inspected and instrumented during the mapper compilation phase.  The
 collection_class callable will be executed once to produce a specimen
@@ -134,13 +134,13 @@ def column_mapped_collection(mapping_spec):
         def keyfunc(value):
             state = instance_state(value)
             m = _state_mapper(state)
-            return m._get_state_attr_by_column(state, cols[0])
+            return m._get_state_attr_by_column(state, state.dict, cols[0])
     else:
         mapping_spec = tuple(cols)
         def keyfunc(value):
             state = instance_state(value)
             m = _state_mapper(state)
-            return tuple(m._get_state_attr_by_column(state, c)
+            return tuple(m._get_state_attr_by_column(state, state.dict, c)
                          for c in mapping_spec)
     return lambda: MappedCollection(keyfunc)
 
@@ -472,6 +472,7 @@ class CollectionAdapter(object):
     """
     def __init__(self, attr, owner_state, data):
         self.attr = attr
+        # TODO: figure out what this being a weakref buys us
         self._data = weakref.ref(data)
         self.owner_state = owner_state
         self.link_to_self(data)
@@ -528,7 +529,11 @@ class CollectionAdapter(object):
         if getattr(obj, '_sa_adapter', None) is not None:
             return getattr(obj, '_sa_adapter')
         elif setting_type == dict:
+            # Py3K
+            #return obj.values()
+            # Py2K
             return getattr(obj, 'itervalues', getattr(obj, 'values'))()
+            # end Py2K
         else:
             return iter(obj)
 
@@ -560,7 +565,9 @@ class CollectionAdapter(object):
 
     def __iter__(self):
         """Iterate over entities in the collection."""
-        return getattr(self._data(), '_sa_iterator')()
+        
+        # Py3K requires iter() here
+        return iter(getattr(self._data(), '_sa_iterator')())
 
     def __len__(self):
         """Count entities in the collection."""
@@ -578,7 +585,7 @@ class CollectionAdapter(object):
 
         """
         if initiator is not False and item is not None:
-            return self.attr.fire_append_event(self.owner_state, item, initiator)
+            return self.attr.fire_append_event(self.owner_state, self.owner_state.dict, item, initiator)
         else:
             return item
 
@@ -591,7 +598,7 @@ class CollectionAdapter(object):
 
         """
         if initiator is not False and item is not None:
-            self.attr.fire_remove_event(self.owner_state, item, initiator)
+            self.attr.fire_remove_event(self.owner_state, self.owner_state.dict, item, initiator)
 
     def fire_pre_remove_event(self, initiator=None):
         """Notify that an entity is about to be removed from the collection.
@@ -600,7 +607,7 @@ class CollectionAdapter(object):
         fire_remove_event().
 
         """
-        self.attr.fire_pre_remove_event(self.owner_state, initiator=initiator)
+        self.attr.fire_pre_remove_event(self.owner_state, self.owner_state.dict, initiator=initiator)
 
     def __getstate__(self):
         return {'key': self.attr.key,
@@ -937,22 +944,23 @@ def _list_decorators():
                 fn(self, index, value)
             else:
                 # slice assignment requires __delitem__, insert, __len__
-                if index.stop is None:
-                    stop = 0
-                elif index.stop < 0:
-                    stop = len(self) + index.stop
-                else:
-                    stop = index.stop
                 step = index.step or 1
-                rng = range(index.start or 0, stop, step)
+                start = index.start or 0
+                if start < 0:
+                    start += len(self)
+                stop = index.stop or len(self)
+                if stop < 0:
+                    stop += len(self)
+                
                 if step == 1:
-                    for i in rng:
-                        del self[index.start]
-                    i = index.start
-                    for item in value:
-                        self.insert(i, item)
-                        i += 1
+                    for i in xrange(start, stop, step):
+                        if len(self) > start:
+                            del self[start]
+                    
+                    for i, item in enumerate(value):
+                        self.insert(i + start, item)
                 else:
+                    rng = range(start, stop, step)
                     if len(value) != len(rng):
                         raise ValueError(
                             "attempt to assign sequence of size %s to "
@@ -979,6 +987,7 @@ def _list_decorators():
         _tidy(__delitem__)
         return __delitem__
 
+    # Py2K
     def __setslice__(fn):
         def __setslice__(self, start, end, values):
             for value in self[start:end]:
@@ -995,7 +1004,8 @@ def _list_decorators():
             fn(self, start, end)
         _tidy(__delslice__)
         return __delslice__
-
+    # end Py2K
+    
     def extend(fn):
         def extend(self, iterable):
             for value in iterable:
@@ -1318,9 +1328,14 @@ class InstrumentedSet(set):
 class InstrumentedDict(dict):
     """An instrumented version of the built-in dict."""
 
+    # Py3K
+    #__instrumentation__ = {
+    #    'iterator': 'values', }
+    # Py2K
     __instrumentation__ = {
         'iterator': 'itervalues', }
-
+    # end Py2K
+    
 __canned_instrumentation = {
     list: InstrumentedList,
     set: InstrumentedSet,
@@ -1337,8 +1352,13 @@ __interfaces = {
           'iterator': '__iter__',
           '_decorators': _set_decorators(), },
     # decorators are required for dicts and object collections.
+    # Py3K
+    #dict: {'iterator': 'values',
+    #       '_decorators': _dict_decorators(), },
+    # Py2K
     dict: {'iterator': 'itervalues',
            '_decorators': _dict_decorators(), },
+    # end Py2K
     # < 0.4 compatible naming, deprecated- use decorators instead.
     None: {}
     }
